@@ -1,5 +1,4 @@
 from django.contrib import admin
-from django.conf import settings
 from django.contrib.comments.models import Comment
 from django.utils.translation import ugettext_lazy as _
 
@@ -16,9 +15,10 @@ class CommentsAdmin(admin.ModelAdmin):
         ),
      )
 
-    list_display = ('name', 'content_type', 'object_pk', 'ip_address', 'is_public', 'is_removed')
+    list_display = ('name', 'content_type', 'object_pk', 'ip_address', 'submit_date', 'is_public', 'is_removed')
     list_filter = ('submit_date', 'site', 'is_public', 'is_removed')
     date_hierarchy = 'submit_date'
+    ordering = ('-submit_date',)
     search_fields = ('comment', 'user__username', 'user_name', 'user_email', 'user_url', 'ip_address')
 
 admin.site.register(Comment, CommentsAdmin)
