@@ -1,17 +1,20 @@
 from django.db.backends import BaseDatabaseClient
 from django.conf import settings
 import os
+import sys
 
 class DatabaseClient(BaseDatabaseClient):
+    executable_name = 'mysql'
+
     def runshell(self):
-        args = ['']
+        args = [self.executable_name]
         db = settings.DATABASE_OPTIONS.get('db', settings.DATABASE_NAME)
         user = settings.DATABASE_OPTIONS.get('user', settings.DATABASE_USER)
         passwd = settings.DATABASE_OPTIONS.get('passwd', settings.DATABASE_PASSWORD)
         host = settings.DATABASE_OPTIONS.get('host', settings.DATABASE_HOST)
         port = settings.DATABASE_OPTIONS.get('port', settings.DATABASE_PORT)
         defaults_file = settings.DATABASE_OPTIONS.get('read_default_file')
-        # Seems to be no good way to set sql_mode with CLI
+        # Seems to be no good way to set sql_mode with CLI.
     
         if defaults_file:
             args += ["--defaults-file=%s" % defaults_file]
@@ -26,4 +29,7 @@ class DatabaseClient(BaseDatabaseClient):
         if db:
             args += [db]
 
-        os.execvp('mysql', args)
+        if os.name == 'nt':
+            sys.exit(os.system(" ".join(args)))
+        else:
+            os.execvp(self.executable_name, args)
