@@ -68,6 +68,11 @@ show_authors = False
 # The name of the Pygments (syntax highlighting) style to use.
 pygments_style = 'trac'
 
+# Sphinx will recurse into subversion configuration folders and try to read  
+# any document file within. These should be ignored. 
+# Note: exclude_dirnames is new in Sphinx 0.5 
+exclude_dirnames = ['.svn']
+
 # Options for HTML output
 # -----------------------
 
@@ -125,7 +130,7 @@ htmlhelp_basename = 'Djangodoc'
 # (source start file, target name, title, author, document class [howto/manual]).
 #latex_documents = []
 latex_documents = [
-  ('index', 'django.tex', 'Django Documentation', 'Django Software Foundation', 'manual'),
+  ('contents', 'django.tex', 'Django Documentation', 'Django Software Foundation', 'manual'),
 ]
 
 # Additional stuff for the LaTeX preamble.
