@@ -103,6 +103,19 @@ class DispatcherTests(unittest.TestCase):
         a_signal.disconnect(fails)
         self._testIsClean(a_signal)
 
+    def testDisconnection(self):
+        receiver_1 = Callable()
+        receiver_2 = Callable()
+        receiver_3 = Callable()
+        a_signal.connect(receiver_1)
+        a_signal.connect(receiver_2)
+        a_signal.connect(receiver_3)
+        a_signal.disconnect(receiver_1)
+        del receiver_2
+        garbage_collect()
+        a_signal.disconnect(receiver_3)
+        self._testIsClean(a_signal)
+
 def getSuite():
     return unittest.makeSuite(DispatcherTests,'test')
 
